@@ -2,7 +2,7 @@
  *
  * err_msgs.h - FLash INTerface
  *
- * Copyright (C) Jan 2013 Mellanox Technologies Ltd. All rights reserved.
+ * Copyright (c) 2013 Mellanox Technologies Ltd.  All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -59,8 +59,8 @@ typedef enum {
 #define FLINT_PARSE_MEM_ERROR                 "Failed to allocate memory for parsing.\n "
 #define FLINT_NO_OPTIONS_FOUND_ERROR          "No options found. "
 #define FLINT_INVALID_COMMAD_ERROR            "Invalid command: %s\n"
-#define FLINT_TOO_MANY_ARGS_ERROR             "Too many arguments. Expected: %d , Recieved: %d\n"
-#define FLINT_TOO_FEW_ARGS_ERROR              "Too few arguments. Expected: %d , Recieved: %d\n"
+#define FLINT_TOO_MANY_ARGS_ERROR             "Too many arguments. Expected: %d , Received: %d\n"
+#define FLINT_TOO_FEW_ARGS_ERROR              "Too few arguments. Expected: %d , Received: %d\n"
 #define FLINT_NO_COMMAND_ERROR                "No command found."
 #define FLINT_OPEN_FWOPS_DEVICE_ERROR         "Cannot open Device: %s. %s\n"
 #define FLINT_OPEN_FWOPS_IMAGE_ERROR          "Cannot open Image: %s. %s\n"
@@ -71,11 +71,12 @@ typedef enum {
 #define FLINT_CMD_ARGS_ERROR                  "Command \"%s\" requires %d arguments, but %d arguments were given\n"
 #define FLINT_CMD_ARGS_ERROR2                 "Command \"%s\" requires at most %d arguments, but %d arguments were given\n"
 #define FLINT_CMD_ARGS_ERROR3                 "Command \"%s\" requires at least %d arguments, but %d arguments were given\n"
+#define FLINT_CMD_ARGS_ERROR4                 "Command \"%s\" requires %d or %d arguments, but %d arguments were given\n"
+#define FLINT_CMD_ARGS_ERROR5                 "Command \"%s\" does not require arguments\n"
 #define FLINT_INVALID_OPTION_ERROR            "Unknown option \"%s\" for the \"%s\" command. you can use %s.\n"
 #define FLINT_INVALID_FLAG_ERROR              "Invalid switch \"%s\" is specified.\n"
-#define FLINT_INVALID_UID_NUM_BX_ERROR        "Number of UIDs on BridgeX should be %d\n"
-#define FLINT_BX_BAD_MAC_FORMAT_ERROR         "Bad mac ( %4.4x%8.8x ) %s. Please re-burn with a valid MACs value.\n"
 #define FLINT_INVALID_FLAG_WITH_FLAG_ERROR    "Cannot specify \"%s\" flag with \"%s\" flag.\n"
+#define FLINT_INVALID_FLAG_WITHOUT_FLAG_ERROR "\"%s\" flag must be specified with \"%s\" flag.\n"
 #define FLINT_INVALID_FLAG_WITH_CMD_ERROR     "Cannot specify flag: %s with Command: %s\n"
 #define FLINT_CMD_VERIFY_ERROR                "FW image verification failed: %s. AN HCA DEVICE CAN NOT BOOT FROM THIS IMAGE.\n"
 #define FLINT_FAILED_QUERY_ERROR              "Failed to query %s: %s.%s\n"
@@ -93,12 +94,14 @@ typedef enum {
 #define FLINT_FLASH_WRITE_ERROR               "Flash write failed. %s\n"
 #define FLINT_INVALID_DATA_ERROR              "Invalid data \"%s\"\n"
 #define FLINT_INVALID_SIZE_ERROR              "Invalid size \"%s\", Length should be 4-bytes aligned.\n"
+#define FLINT_INVALID_ARG_ERROR               "Invalid argument \"%s\"\n"
 #define FLINT_OPEN_FILE_ERROR                 "Cannot open %s: %s\n"
 #define FLINT_WRITE_FILE_ERROR                "Failed to write to %s: %s\n"
 #define FLINT_IO_OPEN_ERROR                   "Failed to open %s: %s\n"
 #define FLINT_IMAGE_READ_ERROR                "Failed to read image. %s\n"
 #define FLINT_READ_ERROR                      "Failed to read from %s. %s\n"
 #define FLINT_WIN_NOT_SUPP_ERROR              "Command \"%s\" is not supported in windows.\n"
+#define FLINT_WIN_ONLY_SUPP_ERROR             "Command  \"%s\" is supported only in windows.\n"
 #define FLINT_GEN_COMMAND_ERROR               "Failed to execute command %s. %s\n"
 #define FLINT_FS3_BB_ERROR                    "bb command is not supported anymore in FS3 image, please use b for burning FS3 image.\n"
 #define FLINT_FS3_BURN_ERROR                  "Burning FS3 image failed: %s\n"
@@ -122,7 +125,17 @@ typedef enum {
 #define FLINT_ROM_QUERY_ERROR                 "Image file rom (%s) query failed. %s\n"
 #define FLINT_WB_FILE_ERROR                   "failed to open file: %s. %s\n"
 #define FLINT_WB_ERROR                        "write Block Failed. %s\n"
-#define FLINT_NO_ZLIB_ERROR					  "Executable was compiled with \"dump files\" option disabled.\n"
+#define FLINT_NO_ZLIB_ERROR                   "Executable was compiled with \"dump files\" option disabled.\n"
+#define FLINT_FLAG_WITH_FLAG_ERROR            "\"%s\" flag must be specified with \"%s\" flag.\n"
+#define FLINT_INVALID_PASSWORD                "Invalid Password.\n"
+#define FLINT_NO_GUID_MAC_FLAGS_ERROR         "Can not set GUIDs/MACs: please run with -uid/-guid/-mac flag.\n"
+#define FLINT_NO_UID_FLAG_ERROR               "Can not set GUIDs/MACs: uid is not specified, please run with -uid flag.\n"
+#define FLINT_CHECKSUM_ERROR                  "Failed to calculate checksum on %s: %s\n"
+#define FLINT_CHECKSUM_MISMATCH_ERROR         "Given checksum: %s does not match the checksum calculated on device FW: %s.\n"
+#define FLINT_CHECKSUM_PARSE_ERROR            "Failed to parse given checksum.\n"
+#define FLINT_CHECKSUM_LEN_ERROR              "MD5 checksum should be exactly 16 bytes long.\n"
+#define FLINT_CHECKSUM_HEX_ERROR              "MD5 checksum should contain only hexadecimal digits.\n"
+#define FLINT_CACHE_IMAGE_ERROR                 "Failed to issue image cache request to driver. %s. make sure Mellanox driver is loaded and working properly.\n"
 
 /**************************
  * Flint Warning Messages
