@@ -31,8 +31,15 @@
  */
 
 
+#if !defined(UEFI_BUILD) && !defined(NO_OPEN_SSL)
+#include <tools_crypto/tools_md5.h>
+#endif
+#include <cibfw_layouts.h>
 #include "fs2_ops.h"
-// #include "flint_ops.h"
+
+#ifdef __WIN__
+#include <win_driver_cif.h>
+#endif
 
 #define PRE_CRC_OUTPUT   "    "
 #define CRC_CHECK_OUTPUT  CRC_CHECK_OLD")"
@@ -45,6 +52,7 @@ bool Fs2Operations::FwInit()
 {
     FwInitCom();
     memset(&_fs2ImgInfo, 0, sizeof(_fs2ImgInfo));
+    _fwImgInfo.fwType = FIT_FS2;
     return true;
 }
 
@@ -87,12 +95,13 @@ bool Fs2Operations::ParseInfoSect(u_int8_t* buff, u_int32_t byteSize) {
         u_int32_t tagId   = __be32_to_cpu(*p) >> 24;
 
         if (offs + tagSize > byteSize) {
-            return errmsg("Info section corrupted: Tag %d (TagId %d, size %d) exceeds Info section size (%d bytes) ",
+            return errmsg(MLXFW_SECTION_TOO_LARGE_ERR, "Info section corrupted: Tag %d (TagId %d, size %d) exceeds Info section size (%d bytes) ",
                           tagNum, tagId, tagSize, byteSize);
         }
         // printf("-D- tagId = (%#x), tagSize = (%#x), offs = %#x\n", tagId, tagSize, offs);
         u_int32_t   tmp;
         const char* str;
+        int size;
         switch (tagId) {
         case II_FwVersion:
             _fwImgInfo.ext_info.fw_ver[0] = u_int16_t(__be32_to_cpu(*(p+1)) >> 16);
@@ -149,6 +158,17 @@ bool Fs2Operations::ParseInfoSect(u_int8_t* buff, u_int32_t byteSize) {
                 _fwImgInfo.ext_info.psid[PSID_LEN] = '\0';
             break;
 
+        case II_PSInfo:
+            str = (const char*)p;
+            str += 4;
+            size = strlen(str) + 1; // include '\0'
+            if (size > PRS_NAME_LEN ) {
+                // ensure '\0' at the end
+                size = PRS_NAME_LEN - 1;
+            }
+            memcpy(_fs2ImgInfo.ext_info.prs_name, str, size);
+            break;
+
         case II_VSD:
             // set psid only if not previosly found in PS
                 str = (const char*)p;
@@ -238,10 +258,10 @@ bool Fs2Operations::ParseInfoSect(u_int8_t* buff, u_int32_t byteSize) {
 
     if (offs != byteSize) {
         if (endFound) {
-            return errmsg("Info section corrupted: Section data size is 0x%x bytes, "
+            return errmsg(MLXFW_SECTION_CORRUPTED_ERR, "Info section corrupted: Section data size is 0x%x bytes, "
                           "but end tag found after 0x%x bytes.", byteSize, offs);
         } else {
-            return errmsg("Info section corrupted: Section data size is 0x%x bytes, "
+            return errmsg(MLXFW_SECTION_CORRUPTED_ERR, "Info section corrupted: Section data size is 0x%x bytes, "
                           "but end tag not found before section end.", byteSize);
         }
     }
@@ -433,12 +453,12 @@ bool Fs2Operations::Fs2Verify(VerifyCallBack verifyCallBackFunc, bool is_striped
 
     // printf("-D- VerifyFs2 = ok\n");
     // Look for image in "physical addresses
-    CntxFindAllImageStart(_ioAccess, cntx_image_start, &cntx_image_num);
+    FindAllImageStart(_ioAccess, cntx_image_start, &cntx_image_num, _cntx_magic_pattern);
     if (cntx_image_num == 0) {
-        return errmsg("No valid image found");
+        return errmsg(MLXFW_NO_VALID_IMAGE_ERR, "No valid image found");
     } else if (cntx_image_num > 2) {
         // This check may be redundant - Maybe ignore if more than 2 images found
-        return errmsg("More than 2 image start locations found at addresses 0x%x, 0x%x and 0x%x. Image may be corrupted.",
+        return errmsg(MLXFW_MULTIPLE_VALID_IMAGES_ERR, "More than 2 image start locations found at addresses 0x%x, 0x%x and 0x%x. Image may be corrupted.",
                       cntx_image_start[0],
                       cntx_image_start[1],
                       cntx_image_start[2]);
@@ -452,8 +472,8 @@ bool Fs2Operations::Fs2Verify(VerifyCallBack verifyCallBackFunc, bool is_striped
 
      // Verify the images:
      for (u_int32_t i = 0; i < cntx_image_num; i++) {
-         bool      fs_en;
-         u_int32_t log2chunk_size;
+         bool      fs_en = false;
+         u_int32_t log2chunk_size = 0;
          u_int32_t buff[FS2_BOOT_START / 4];
 
          _ioAccess->get_image_crc().clear();
@@ -683,7 +703,7 @@ bool Fs2Operations::Fs2Query () {
     }
 
     if (info_ptr_cs) {
-        return errmsg("Failed to read Info Section - Bad checksum for Info section pointer (%08x). Probably the image is corrupted.", info_ptr);
+        return errmsg(MLXFW_BAD_CHECKSUM_ERR, "Failed to read Info Section - Bad checksum for Info section pointer (%08x). Probably the image is corrupted.", info_ptr);
     }
 
     info_ptr = info_ptr_ba.range(23,0);
@@ -692,7 +712,7 @@ bool Fs2Operations::Fs2Query () {
 
         info_ptr += im_start;
         if (info_ptr >= _ioAccess->get_size()) {
-            return errmsg("Failed to read Info Section - Info section pointer (%08x) too large. Probably the image is corrupted.", info_ptr);
+            return errmsg(MLXFW_SECTION_TOO_LARGE_ERR, "Failed to read Info Section - Info section pointer (%08x) too large. Probably the image is corrupted.", info_ptr);
         }
         READ4((*_ioAccess), info_ptr - 3*sizeof(u_int32_t), &info_size, "Info section size");
         TOCPU1(info_size);
@@ -721,7 +741,7 @@ bool Fs2Operations::Fs2IntQuery(bool readRom, bool isStripedImage)
     } else {
         initSectToRead(H_LAST);
     }
-    if (!Fs2Verify(NULL, isStripedImage)) {
+    if (!Fs2Verify((VerifyCallBack)NULL, isStripedImage)) {
         initSectToRead(H_LAST);
         return false;
     }
@@ -730,6 +750,11 @@ bool Fs2Operations::Fs2IntQuery(bool readRom, bool isStripedImage)
         return false;
     }
     _fwImgInfo.ext_info.chip_type = getChipType();
+
+    // get running FW version
+    if (_ioAccess->is_flash() && _fwImgInfo.ext_info.chip_type != CT_UNKNOWN) {
+        getRunningFwVer();
+    }
     return true;
 }
 
@@ -745,6 +770,46 @@ bool Fs2Operations::FwQuery(fw_info_t *fwInfo, bool readRom, bool isStripedImage
     return true;
 }
 
+#define CX3_FW_VER_CR_ADDR 0x1f064
+#define SX_FW_VER_CR_ADDR 0x60040
+
+bool Fs2Operations::getRunningFwVer()
+{
+#ifndef UEFI_BUILD
+    u_int32_t fwVerBaseAddr = 0x0;
+    u_int32_t mflags;
+    struct cibfw_FW_VERSION fwVer;
+    u_int8_t buff[CIBFW_FW_VERSION_SIZE] = {0};
+    memset(&fwVer, 0, sizeof(fwVer));
+    // make sure its not mellanox OS
+    if (mget_mdevs_flags(((Flash*)_ioAccess)->getMfileObj(), &mflags)) {
+        return errmsg("Failed to get device access type");
+    }
+    if (mflags & MDEVS_MLNX_OS) {
+        return true;
+    }
+    switch (_fwImgInfo.ext_info.chip_type) {
+    case CT_CONNECTX:
+        fwVerBaseAddr = CX3_FW_VER_CR_ADDR;
+        break;
+    case CT_SWITCHX:
+        fwVerBaseAddr = SX_FW_VER_CR_ADDR;
+        break;
+    default:
+        return errmsg("Unsupported chip type.");
+    }
+
+    if (mread_buffer(((Flash*)_ioAccess)->getMfileObj(), fwVerBaseAddr, buff, CIBFW_FW_VERSION_SIZE) != CIBFW_FW_VERSION_SIZE) {
+        return errmsg("Failed to extract FW version from device. CR_ERROR\n");
+    }
+    cibfw_FW_VERSION_unpack(&fwVer, buff);
+    _fwImgInfo.ext_info.running_fw_ver[0] = fwVer.MAJOR;
+    _fwImgInfo.ext_info.running_fw_ver[1] = fwVer.MINOR;
+    _fwImgInfo.ext_info.running_fw_ver[2] = fwVer.SUBMINOR;
+#endif
+    return true;
+}
+
 #define MY_MAX(a, b) ((a) > (b) ? (a) : (b))
 #define CX_DFLT_SECTOR_SIZE 0x10000
 #define SX_DFLT_SECTOR_SIZE 0x1000
@@ -759,7 +824,7 @@ u_int32_t Fs2Operations::getDefaultSectorSz()
     // an older mlxburn the fw_sector_size wount be available
     u_int32_t devid = _ioAccess->get_dev_id();
     switch (devid) {
-    case CX_HW_ID:
+    case CX2_HW_ID:
     case CX3_HW_ID:
     case CX3_PRO_HW_ID:
         return CX_DFLT_SECTOR_SIZE;
@@ -834,14 +899,11 @@ bool Fs2Operations::UpdateFullImageCRC(u_int32_t* buff, u_int32_t size, bool bla
     return true;
 }
 #define ERASE_MESSAGE "    Please erase them by using the command: \"" MLXCONFIG_CMD " " ERASE_CMD "\" and then re-burn"
-//TODO: remove pre_message from function def
+
 bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
-                                  bool      allow_nofs,
-                                  const char* pre_message,
-                                  ProgressCallBack progressFunc) {
-    //we do not use pre_message, avoid warrning
-	//TODO: remove pre_message from function definition
-    (void)pre_message;
+                                                                   ExtBurnParams& burnParams) {
+    bool allow_nofs = !burnParams.burnFailsafe;
+    ProgressCallBack progressFunc = burnParams.progressFunc;
 
     Flash  *f = (Flash*)(this->_ioAccess);
     FImage *fim = (FImage*)(imageOps._ioAccess);
@@ -861,11 +923,11 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
     // TODO: Do we need the verify ORENK
     if (!allow_nofs) {
         if (!imageOps._fwImgInfo.ext_info.is_failsafe) {
-            return errmsg("The given image is not a failsafe image");
+            return errmsg(MLXFW_IMAGE_NOT_FS_ERR, "The given image is not a failsafe image");
         }
 
         if (_fwImgInfo.cntxLog2ChunkSize != imageOps._fwImgInfo.cntxLog2ChunkSize) {
-            return errmsg("Failsafe chunk sizes in flash (0x%x) and in image (0x%x) are not the same.",
+            return errmsg(MLXFW_FS_INFO_MISMATCH_ERR, "Failsafe chunk sizes in flash (0x%x) and in image (0x%x) are not the same.",
                           1 << _fwImgInfo.cntxLog2ChunkSize,
                           1 << imageOps._fwImgInfo.cntxLog2ChunkSize);
         }
@@ -882,7 +944,7 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
         const char *image_type = (imageOps._fwImgInfo.ext_info.is_failsafe) ? "failsafe" : "non-failsafe";
         const char *note_str   = (imageOps._fwImgInfo.ext_info.is_failsafe) ? " - half of total flash size" : " - total flash size";
 
-        return errmsg("Size of %s image (0x%x) is greater than max %s image size (0x%x%s)",
+        return errmsg(MLXFW_IMAGE_TOO_LARGE_ERR, "Size of %s image (0x%x) is greater than max %s image size (0x%x%s)",
                  image_type, imageOps._fwImgInfo.ext_info.image_size, image_type, max_image_size, note_str);
 
     }
@@ -913,7 +975,7 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
     // Write new signature
     if (!f->write(0, data8, 16, true)) {
         //return false;
-        return errmsg("Flash write failed. %s", f->err());
+        return errmsg(MLXFW_FLASH_WRITE_ERR, "Flash write failed. %s", f->err());
     }
     bool boot_address_was_updated = true;
     // Write new image start address to crspace (for SW reset)
@@ -930,13 +992,13 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
             u_int32_t cntx_image_start[CNTX_START_POS_SIZE];
             u_int32_t cntx_image_num;
 
-            CntxFindAllImageStart(_ioAccess, cntx_image_start, &cntx_image_num);
+            FindAllImageStart(_ioAccess, cntx_image_start, &cntx_image_num, _cntx_magic_pattern);
             // Address convertor is disabled now - use phys addresses
             for (u_int32_t i = 0; i < cntx_image_num; i++) {
                 if (cntx_image_start[i] != new_image_start) {
                     if (!f->write(cntx_image_start[i], &zeroes, sizeof(zeroes), true)) {
                         //return false;
-                        return errmsg("Flash write failed. %s", f->err());
+                        return errmsg(MLXFW_FLASH_WRITE_ERR, "Flash write failed. %s", f->err());
                     }
                 }
             }
@@ -945,23 +1007,41 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
             f->set_address_convertor(imageOps._fwImgInfo.cntxLog2ChunkSize, is_curr_image_in_odd_chunks);
             if (!f->write(0, &zeroes, sizeof(zeroes), true)) {
                 //return false;
-                return errmsg("Flash write failed. %s", f->err());
+                return errmsg(MLXFW_FLASH_WRITE_ERR, "Flash write failed. %s", f->err());
             }
         }
     }
     if (boot_address_was_updated == false) {
         report_warn("Failed to update FW boot address. Power cycle the device in order to load the new FW.\n");
     }
-
+    // on windows send caching command to driver (best effort)
+#ifdef __WIN__
+    if (!burnParams.skipCiReq) {
+        int rc;
+        mf_release_semaphore(((Flash*)_ioAccess)->getMflashObj());
+        rc = wdcif_send_image_cache_request(((Flash*)_ioAccess)->getMfileObj());
+        switch (rc) {
+        case WDCIF_STATUS_SUCCESS :
+            burnParams.burnStatus.imageCachedSuccessfully=true;
+            break;
+        case WDCIF_STATUS_OPERATION_NOT_SUPPORTED_BY_DRIVER :
+        case WDCIF_STATUS_OPERATION_NOT_SUPPORTED_BY_DEVICE :
+        case WDCIF_STATUS_UNSUPPORTED_DEVICE :
+        case WDCIF_STATUS_UNSUPPORTED_ACCESS_TYPE :
+        case WDCIF_STATUS_FAILED_TO_RETRIEVE_DRIVER_HANDLE :
+            break;
+        default:
+            report_warn("Failed to Issue cache request to driver. next driver load may take more time.\n");
+        }
+    }
+#endif
     return true;
 }
 
 
 bool Fs2Operations::preFS2PatchGUIDs(bool      patch_macs,
-        bool      patch_uids,
         bool      user_guids,
         bool      user_macs,
-        bool      user_uids,
         guid_t    new_guids[MAX_GUIDS],
         guid_t    old_guids[MAX_GUIDS],
         guid_t    **used_guids_p,
@@ -974,98 +1054,66 @@ bool Fs2Operations::preFS2PatchGUIDs(bool      patch_macs,
         // if only guids or only macs are specified by user, keep the other
         // as currently set of flash. This is in order to simplify transitions between
         // burning IB and ETH FW.
-        if (!patch_uids) {
-            if (old_guids && !user_guids) {
-                for (int i = 0; i < GUIDS; i++) {
-                    new_guids[i] = old_guids[i];
-                }
+        if (old_guids && !user_guids) {
+            for (int i = 0; i < GUIDS; i++) {
+                new_guids[i] = old_guids[i];
             }
+        }
 
-            if (old_guids && !user_macs) {
-                for (int i = GUIDS; i < MAX_GUIDS; i++) {
-                    new_guids[i] = old_guids[i];
-                }
+        if (old_guids && !user_macs) {
+            for (int i = GUIDS; i < MAX_GUIDS; i++) {
+                new_guids[i] = old_guids[i];
             }
         }
         *used_guids_p = new_guids;
     }
     used_guids = *used_guids_p;
 
-    if (!patch_uids) {
-        if (patch_macs) {
+    if (patch_macs) {
 
-            // To ease upgrade from 4 GUIDS format to 4+2 format, or to move from IB to ETH,
-            // if macs are not
-            // explicitly set in flash, they are derived from the GUIDs according to
-            // Mellanox methodology - 48 bit MAC == 64 bit GUID without the middle 16 bits.
+        // To ease upgrade from 4 GUIDS format to 4+2 format, or to move from IB to ETH,
+        // if macs are not
+        // explicitly set in flash, they are derived from the GUIDs according to
+        // Mellanox methodology - 48 bit MAC == 64 bit GUID without the middle 16 bits.
 
-            if (old_guids && ((num_of_old_guids == 4) ||
-                              (num_of_old_guids == 6 &&
-                               (old_guids[GUIDS  ].h & 0xffff)     == 0xffff     &&
-                               (old_guids[GUIDS  ].l & 0xffffffff) == 0xffffffff &&
-                               (old_guids[GUIDS+1].h & 0xffff)     == 0xffff     &&
-                               (old_guids[GUIDS+1].l & 0xffffffff) == 0xffffffff))) {
-                for (int i = 0 ; i < MACS; i++) {
-                    u_int64_t mac  =  old_guids[i+1].h >> 8;
-                    mac <<= 24;
-                    mac |= (old_guids[i+1].l & 0xffffff);
+        if (old_guids && ((num_of_old_guids == 4) ||
+                          (num_of_old_guids == 6 &&
+                           (old_guids[GUIDS  ].h & 0xffff)     == 0xffff     &&
+                           (old_guids[GUIDS  ].l & 0xffffffff) == 0xffffffff &&
+                           (old_guids[GUIDS+1].h & 0xffff)     == 0xffff     &&
+                           (old_guids[GUIDS+1].l & 0xffffffff) == 0xffffffff))) {
+            for (int i = 0 ; i < MACS; i++) {
+                u_int64_t mac  =  old_guids[i+1].h >> 8;
+                mac <<= 24;
+                mac |= (old_guids[i+1].l & 0xffffff);
 
-                    old_guids[GUIDS+i].h = u_int32_t(mac >> 32);
-                    old_guids[GUIDS+i].l = u_int32_t(mac  & 0xffffffff);
+                old_guids[GUIDS+i].h = u_int32_t(mac >> 32);
+                old_guids[GUIDS+i].l = u_int32_t(mac  & 0xffffffff);
 
-                    // printf("-D- Guid " GUID_FORMAT " to MAC "MAC_FORMAT"\n", old_guids[i+1].h, old_guids[i+1].l, old_guids[i+GUIDS].h,old_guids[i+GUIDS].l  );
-                }
-            }
-
-            guid_t* macs = &used_guids[4];
-
-            for (int i = 0 ; i < MACS ; i++) {
-                u_int64_t mac = (((u_int64_t)macs[i].h) << 32) | macs[i].l;
-                if (!_burnBlankGuids && !CheckMac(mac)) {
-                    return errmsg("Bad mac (" MAC_FORMAT ") %s: %s. Please re-burn with a valid -mac flag value.", macs[i].h,
-                           macs[i].l,
-                           user_macs ? "given" : "found on flash", err());
-                }
-
-            }
-        }
-    } else {
-        if (!_burnBlankGuids) {
-            for (int i = 0; i < BX_SLICES_NUM; i++ ) {
-                if (CheckBxMacsFormat(used_guids, i, user_uids) == false) {
-                    return false;
-                }
+                // printf("-D- Guid " GUID_FORMAT " to MAC "MAC_FORMAT"\n", old_guids[i+1].h, old_guids[i+1].l, old_guids[i+GUIDS].h,old_guids[i+GUIDS].l  );
             }
         }
 
+        guid_t* macs = &used_guids[4];
+
+        for (int i = 0 ; i < MACS ; i++) {
+            u_int64_t mac = (((u_int64_t)macs[i].h) << 32) | macs[i].l;
+            if (!_burnBlankGuids && !CheckMac(mac)) {
+                return errmsg("Bad mac (" MAC_FORMAT ") %s: %s. Please re-burn with a valid -mac flag value.", macs[i].h,
+                       macs[i].l,
+                       user_macs ? "given" : "found on flash", err());
+            }
+
+        }
     }
-    // Avoid warnings
-    user_uids = true;
 
     return true;
 }
 
 
-bool Fs2Operations::CheckBxMacsFormat(guid_t* guids, int index, int user_uids)
-{
-    int base;
-    base = index * BX_SLICE_GUIDS + BI_IMACS;
-    for (int i = base; i < base + BX_MACS; i++) {
-         u_int64_t mac = (((u_int64_t)guids[i].h) << 32) | guids[i].l;
-         if (!CheckMac(mac)) {
-             return errmsg("Bad mac (" MAC_FORMAT ") %s: %s. Please re-burn with a valid MACs flag value.", guids[i].h,
-                     guids[i].l,
-                     user_uids ? "given" : "found on flash", err());
-
-         }
-     }
-     return true;
-}
-
-////////////////////////////////////////////////////////////////////////
 void Fs2Operations::patchGUIDsSection(u_int32_t *buf, u_int32_t ind, guid_t guids[MAX_GUIDS], int nguids)
 {
-    u_int32_t       new_buf[MAX_GUIDS*2];
+    u_int32_t       new_buf[MAX_GUIDS*2] = {0};
 
     // Form new GUID section
     for (u_int32_t i=0; i<(u_int32_t)nguids; i++) {
@@ -1092,10 +1140,8 @@ void Fs2Operations::patchGUIDsSection(u_int32_t *buf, u_int32_t ind, guid_t guid
 
 bool Fs2Operations::patchGUIDs (Fs2Operations&   imageOps,
                              bool      patch_macs,
-                             bool      patch_uids,
                              bool      user_guids,
                              bool      user_macs,
-                             bool      user_uids,
                              guid_t    new_guids[MAX_GUIDS],
                              guid_t    old_guids[MAX_GUIDS],
                              u_int32_t num_of_old_guids)
@@ -1104,17 +1150,13 @@ bool Fs2Operations::patchGUIDs (Fs2Operations&   imageOps,
     u_int32_t       *buf = ((FImage*)imageOps._ioAccess)->getBuf();
 
     // Call common function
-    if (!preFS2PatchGUIDs(patch_macs, patch_uids, user_guids, user_macs, user_uids, new_guids, old_guids, &used_guids, num_of_old_guids)) {
+    if (!preFS2PatchGUIDs(patch_macs, user_guids, user_macs, new_guids, old_guids, &used_guids, num_of_old_guids)) {
         return false;
     }
     // Path GUIDs section
     if (imageOps._fs2ImgInfo.guidPtr) {
         patchGUIDsSection(buf, imageOps._fwImgInfo.imgStart + imageOps._fs2ImgInfo.guidPtr, used_guids, imageOps._fs2ImgInfo.ext_info.guid_num);
     }
-
-
-    // Avoid warnings
-    user_uids = true;
     return true;
 }
 
@@ -1343,7 +1385,7 @@ bool Fs2Operations::IntegrateDevRomInImage(Fs2Operations &imageOps)
     ((FImage*)(imageOps._ioAccess))->open((u_int32_t*)(&new_data[0]), actual_image_size);
 
 
-    if (!imageOps.FwVerify(NULL) || !imageOps.Fs2IntQuery()) {
+    if (!imageOps.FwVerify((VerifyCallBack)NULL) || !imageOps.Fs2IntQuery()) {
         return errmsg("Internal error: verify/query of image after integrating ROM failed\n");
     }
 
@@ -1354,7 +1396,7 @@ bool Fs2Operations::IntegrateDevRomInImage(Fs2Operations &imageOps)
 bool Fs2Operations::Fs2Burn(Fs2Operations &imageOps, ExtBurnParams& burnParams)
 {
     if (imageOps.FwType() != FIT_FS2) {
-        return errmsg("FW image type is not FS2\n");
+        return errmsg(MLXFW_IMAGE_FORMAT_ERR, "FW image type is not FS2\n");
     }
 
     if (!imageOps.Fs2IntQuery()) {
@@ -1368,25 +1410,21 @@ bool Fs2Operations::Fs2Burn(Fs2Operations &imageOps, ExtBurnParams& burnParams)
    }
 
     // Check Matching device ID
-    // HACK
-#ifndef UEFI_BUILD // NO Device ID here..
-
     if (!burnParams.noDevidCheck) {
         if (imageOps._fs2ImgInfo.infoOffs[II_HwDevsId]) {
              if (!CheckMatchingHwDevId(_ioAccess->get_dev_id(),
                                          _ioAccess->get_rev_id(),
                                          imageOps._fwImgInfo.supportedHwId,
                                          imageOps._fwImgInfo.supportedHwIdNum)) {
-                 return errmsg("Device/Image mismatch: %s\n",this->err( ));
+                 return errmsg(MLXFW_DEVICE_IMAGE_MISMATCH_ERR, "Device/Image mismatch: %s\n",this->err( ));
              }
          } else if (imageOps._fs2ImgInfo.infoOffs[II_DeviceType]) {
              if (!CheckMatchingDevId(_ioAccess->get_dev_id(), imageOps._fwImgInfo.ext_info.dev_type)) {
-                 return errmsg("Device/Image mismatch: %s\n",this->err());
+                 return errmsg(MLXFW_DEVICE_IMAGE_MISMATCH_ERR, "Device/Image mismatch: %s\n",this->err());
              }
          }
 
     }
-#endif
 
     // Check PSID
     if (devIntQueryRes && !CheckPSID(imageOps, burnParams.allowPsidChange)) {
@@ -1406,24 +1444,18 @@ bool Fs2Operations::Fs2Burn(Fs2Operations &imageOps, ExtBurnParams& burnParams)
         }
     }
 
-
-
-
     // Guids patch
     _burnBlankGuids = burnParams.blankGuids;
-    bool  isGuidsSpecified  =  burnParams.userMacsSpecified || burnParams.userGuidsSpecified ||
-            burnParams.userUidsSpecified;
+    bool  isGuidsSpecified  =  burnParams.userMacsSpecified || burnParams.userGuidsSpecified;
     if (isGuidsSpecified) {
         // Get the GUIDS/MACsUIDs from the user input
-        bool isBridgeX = (_fwImgInfo.ext_info.chip_type == CT_BRIDGEX);
-        bool isMacAvailable = Fs2IsMacAvailable();
-        if (!patchGUIDs(imageOps, isMacAvailable, isBridgeX, burnParams.userGuidsSpecified, burnParams.userMacsSpecified, burnParams.userUidsSpecified,
+        if (!patchGUIDs(imageOps, true, burnParams.userGuidsSpecified, burnParams.userMacsSpecified,
                 (guid_t*)(&(burnParams.userUids[0])), _fs2ImgInfo.ext_info.guids, _fs2ImgInfo.ext_info.guid_num)) {
             return false;
         }
     } else if (!burnParams.useImageGuids) {
         // Get the GUIDS/MACsUIDs from the device
-        if (!patchGUIDs(imageOps, true, false, false, false, false, (guid_t*)NULL,
+        if (!patchGUIDs(imageOps, true, false, false, (guid_t*)NULL,
                 _fs2ImgInfo.ext_info.guids, _fs2ImgInfo.ext_info.guid_num)) {
             return false;
         }
@@ -1434,12 +1466,9 @@ bool Fs2Operations::Fs2Burn(Fs2Operations &imageOps, ExtBurnParams& burnParams)
             return false;
         }
     }
-
-    // TODO: EXp Rom checks - Take from flash ..??
-    return Fs2FailSafeBurn(imageOps, !burnParams.burnFailsafe, "", burnParams.progressFunc);
+    return Fs2FailSafeBurn(imageOps, burnParams);
 }
 
-
 bool Fs2Operations::Fs2IsMacAvailable()
 {
 
@@ -1448,6 +1477,7 @@ bool Fs2Operations::Fs2IsMacAvailable()
     }
     return true;
 }
+
 bool Fs2Operations::FwBurn(FwOperations *imageOps, u_int8_t forceVersion, ProgressCallBack progressFunc)
 {
     if (imageOps == NULL) {
@@ -1472,12 +1502,11 @@ bool Fs2Operations::FwBurnAdvanced(FwOperations *imageOps, ExtBurnParams& burnPa
 
 bool Fs2Operations::FwBurnBlock(FwOperations *imageOps, ProgressCallBack progressFunc)
 {
-    FImage *fim = (FImage*)((Fs2Operations*)imageOps)->_ioAccess;
-
     if (imageOps == NULL) {
         return errmsg("bad parameter is given to FwBurnBlock\n");
     }
 
+    FImage *fim = (FImage*)((Fs2Operations*)imageOps)->_ioAccess;
     return writeImage(progressFunc, 0, fim->getBuf(), fim->getBufLength());
 }
 
@@ -1493,6 +1522,14 @@ bool Fs2Operations::FwReadRom(std::vector<u_int8_t>& romSect)
     return true;
 }
 
+bool Fs2Operations::FwSetMFG(fs3_uid_t baseGuid, PrintCallBack callBackFunc)
+{
+    // avoid compiler warrnings
+    (void)baseGuid;
+    (void)callBackFunc;
+    return errmsg("This command is not supported for FS2 FW image.");
+}
+
 bool Fs2Operations::FwSetMFG(guid_t baseGuid, PrintCallBack callBackFunc)
 {
     // avoid compiler warrnings
@@ -1501,13 +1538,13 @@ bool Fs2Operations::FwSetMFG(guid_t baseGuid, PrintCallBack callBackFunc)
     return errmsg("This command is not supported for FS2 FW image.");
 }
 
-bool Fs2Operations::FwGetSection (u_int32_t sectType, std::vector<u_int8_t>& sectInfo)
+bool Fs2Operations::FwGetSection (u_int32_t sectType, std::vector<u_int8_t>& sectInfo, bool stripedImage)
 {
     if (sectType != H_FW_CONF && sectType != H_HASH_FILE) {
         return errmsg("Hash File section not found in the given image.");
     }
     initSectToRead(sectType);
-    if (!Fs2Verify()) {
+    if (!Fs2Verify((VerifyCallBack)NULL, stripedImage)) {
         return false;
     }
     if (sectType == H_FW_CONF) {
@@ -1541,6 +1578,10 @@ bool Fs2Operations::ModifyVSDSection(const char *vsd, ProgressCallBack callBackF
 
 bool Fs2Operations::ReburnNewImage(u_int8_t *data, const char *feature_name, ProgressCallBack callBackFunc)
 {
+    ExtBurnParams burnParams;
+    burnParams.progressFunc = callBackFunc;
+    burnParams.burnFailsafe = false;
+
     u_int32_t length        = _fwImgInfo.lastImageAddr;
     //char burn_str[100];
     bool is_image = (_fname != NULL);
@@ -1553,11 +1594,11 @@ bool Fs2Operations::ReburnNewImage(u_int8_t *data, const char *feature_name, Pro
 
     // Verify the new image and exit if it's not VALID.
     if (!((Fs2Operations*)newOps)->Fs2IntQuery()) {
-        return errmsg("Internal error: The prepared image After modifying the %s is corrupted: %s\n", feature_name, newOps->err());
+        return errmsg(MLXFW_IMAGE_CORRUPTED_ERR, "Internal error: The prepared image After modifying the %s is corrupted: %s\n", feature_name, newOps->err());
     }
     if (!is_image) {
         // Modify the flash
-        if (!Fs2FailSafeBurn(*((Fs2Operations*)newOps), true, NULL, callBackFunc)) {
+        if (!Fs2FailSafeBurn(*((Fs2Operations*)newOps), burnParams)) {
             return false;
         }
     } else {
@@ -1573,9 +1614,11 @@ bool Fs2Operations::ReburnNewImage(u_int8_t *data, const char *feature_name, Pro
         packStripedImageData(striped_data, data, length, striped_length, needs_repack, _fwImgInfo.cntxLog2ChunkSize);
 
         // Re-write the image to the file.
-        if (!WriteImageToFile(_fname, striped_data, striped_length)) {
+        if (!((FImage*)_ioAccess)->write(0, striped_data, striped_length)) {
+            delete[] striped_data;
             return false;
        }
+        delete[] striped_data;
     }
     return true;
 }
@@ -1711,7 +1754,7 @@ bool Fs2Operations::Fs2SetGuidsForBlank(sg_params_t& sgParam)
         u_int32_t new_crc_dw = CPUTO1(old_dw);
 
         if (!((Flash*)_ioAccess)->write(IMG_CRC_OFF, &new_crc_dw, 4, true)) {
-            return errmsg("flash write error (%s).", _ioAccess->err());
+            return errmsg(MLXFW_FLASH_WRITE_ERR, "flash write error (%s).", _ioAccess->err());
         }
     }
     return true;
@@ -1722,11 +1765,10 @@ bool Fs2Operations::Fs2SetGuids(sg_params_t& sgParam, PrintCallBack callBackFunc
     // avoid compiler warrnings
     (void)callBackFunc;
     //
-    bool ib_dev, eth_dev, bx_dev;
+    bool ib_dev, eth_dev;
     // Get the FW types
-    bx_dev = _fwImgInfo.ext_info.chip_type == CT_BRIDGEX;
     SetDevFlags(_fwImgInfo.ext_info.chip_type, _fwImgInfo.ext_info.dev_type, FIT_FS2, ib_dev, eth_dev);
-    guid_t* old_guids = _fwImgInfo.imageOk ? _fs2ImgInfo.ext_info.guids : NULL;
+    guid_t* old_guids = _fwImgInfo.imageOk ? _fs2ImgInfo.ext_info.guids : (guid_t*)NULL;
     guid_t* used_guids;
 
     // Patch the GUIDs and prints any needed warnings
@@ -1734,7 +1776,7 @@ bool Fs2Operations::Fs2SetGuids(sg_params_t& sgParam, PrintCallBack callBackFunc
         //resize our user guids vector to MAX_GUIDS
     sgParam.userGuids.resize(MAX_GUIDS);
 
-    if (!preFS2PatchGUIDs(eth_dev, bx_dev, sgParam.guidsSpecified, sgParam.macsSpecified, sgParam.uidsSpecified, &sgParam.userGuids[0],
+    if (!preFS2PatchGUIDs(eth_dev, sgParam.guidsSpecified, sgParam.macsSpecified, &sgParam.userGuids[0],
                         old_guids, &used_guids, _fs2ImgInfo.ext_info.guid_num)) {
         return false;
     }
@@ -1768,6 +1810,10 @@ bool Fs2Operations::FwSetGuids(sg_params_t& sgParam, PrintCallBack callBackFunc,
 
 bool Fs2Operations::FwBurnRom(FImage* romImg, bool ignoreProdIdCheck, bool ignoreDevidCheck, ProgressCallBack progressFunc)
 {
+    ExtBurnParams burnParams;
+    burnParams.progressFunc = progressFunc;
+    burnParams.burnFailsafe = false;
+
     // we dont support adding rom to an image just yet
     if (!_ioAccess->is_flash()) {
         return errmsg("Burn ROM not supported for FS2 image.");
@@ -1776,9 +1822,9 @@ bool Fs2Operations::FwBurnRom(FImage* romImg, bool ignoreProdIdCheck, bool ignor
 
     u_int32_t cntx_image_start[CNTX_START_POS_SIZE];
     u_int32_t cntx_image_num;
-    CntxFindAllImageStart(romImg, cntx_image_start, &cntx_image_num);
+    FindAllImageStart(romImg, cntx_image_start, &cntx_image_num, _cntx_magic_pattern);
     if (cntx_image_num != 0) {
-        return errmsg("Expecting an expansion ROM image, Recieved Mellanox FW image.");
+        return errmsg("Expecting an expansion ROM image, Received Mellanox FW image.");
     }
 
     if (!Fs2IntQuery()) {
@@ -1842,7 +1888,7 @@ bool Fs2Operations::FwBurnRom(FImage* romImg, bool ignoreProdIdCheck, bool ignor
         return false;
     }
 
-    bool rc = Fs2FailSafeBurn(*((Fs2Operations*)newOps), true, "Burning ROM image", progressFunc);
+    bool rc = Fs2FailSafeBurn(*((Fs2Operations*)newOps), burnParams);
     newOps->FwCleanUp();
     delete newOps;
     return rc;
@@ -1850,6 +1896,10 @@ bool Fs2Operations::FwBurnRom(FImage* romImg, bool ignoreProdIdCheck, bool ignor
 
 bool Fs2Operations::FwDeleteRom(bool ignoreProdIdCheck, ProgressCallBack progressFunc)
 {
+    ExtBurnParams burnParams;
+    burnParams.progressFunc = progressFunc;
+    burnParams.burnFailsafe = false;
+
     // we dont support delete rom in FS2 image yet
     if (!_ioAccess->is_flash()) {
         return errmsg("Delete ROM not supported for FS2 image.");
@@ -1886,7 +1936,7 @@ bool Fs2Operations::FwDeleteRom(bool ignoreProdIdCheck, ProgressCallBack progres
     }
     int new_image_size;
     if(!UpdateRomInImage((u_int8_t*)(&new_data[0]), (u_int8_t*)(&data[0]),
-                             NULL, 0, &new_image_size)) {
+                             (u_int8_t*)NULL, 0, &new_image_size)) {
         return errmsg("%s", _ioAccess->err());
     }
 
@@ -1903,7 +1953,7 @@ bool Fs2Operations::FwDeleteRom(bool ignoreProdIdCheck, ProgressCallBack progres
         delete newOps;
         return false;
     }
-    bool rc = Fs2FailSafeBurn(*((Fs2Operations*)newOps), true, "Removing ROM image", progressFunc);
+    bool rc = Fs2FailSafeBurn(*((Fs2Operations*)newOps), burnParams);
     newOps->FwCleanUp();
     delete newOps;
     return rc;
@@ -2005,6 +2055,7 @@ bool Fs2Operations::FwResetNvData()
 			//_fs2ImgInfo.ext_info.config_sectors, _fs2ImgInfo.ext_info.config_pad, fwSectorSz,configBaseAddr, availFlashSize);
 
 	//erase addresses : [configBaseAddr..AvailFlashSize]
+    ((Flash*)_ioAccess)->set_flash_working_mode(Flash::Fwm_Default);
     u_int32_t sectorSize = _ioAccess->get_sector_size();
     u_int32_t configEndAddr = availFlashSize - (_fs2ImgInfo.ext_info.config_pad * fwSectorSz);
 	for (u_int32_t eraseAddr=configBaseAddr; eraseAddr < configEndAddr; eraseAddr+=sectorSize ) {
@@ -2031,3 +2082,46 @@ bool Fs2Operations::FwShiftDevData(PrintCallBack progressFunc)
     (void)progressFunc;
     return errmsg("Shifting device data sections is not supported in FS2 image format.");
 }
+
+const char* Fs2Operations::FwGetResetRecommandationStr()
+{
+    if (!_devName) {// not an mst device
+        return (const char*)NULL;
+    }
+    return (const char*)NULL;
+}
+
+bool Fs2Operations::FwCalcMD5(u_int8_t md5sum[16])
+{
+#if defined(UEFI_BUILD) || defined(NO_OPEN_SSL)
+    (void)md5sum;
+    return errmsg("Operation not supported");
+#else
+    // no support for striped image ATM
+    if (!Fs2IntQuery(true, false)) {
+        return false;
+    }
+    if (_fwImgInfo.ext_info.is_failsafe && _fwImgInfo.actuallyFailsafe) {
+        _ioAccess->set_address_convertor(_fwImgInfo.cntxLog2ChunkSize, _fwImgInfo.imgStart != 0);
+    } else {
+        _ioAccess->set_address_convertor(0, 0);
+    }
+    // read entire flash
+    std::vector<u_int8_t> md5buff;
+    md5buff.resize(_fwImgInfo.lastImageAddr);
+    READBUF((*_ioAccess), 0, &md5buff[0], _fwImgInfo.lastImageAddr, "Calculate MD5");
+    // mask out image CRC, GUIDs, VSD, ImageInfo CRC
+    // full image CRC DW
+    memset(&md5buff[0x20], 0xff, 4);
+    // GUIDs
+    memset(&md5buff[_fs2ImgInfo.guidPtr - 0x10], 0xff, 0x44);
+    // VSD
+    memset(&md5buff[_fs2ImgInfo.infoSectPtr + _fs2ImgInfo.infoOffs[II_VSD]], 0xff, 0xd0);
+    // Image Info CRC
+    u_int32_t infoSectSize = __be32_to_cpu(*(u_int32_t*)&md5buff[_fs2ImgInfo.infoSectPtr - sizeof(GPH) + 0x4]) << 2;
+    memset(&md5buff[_fs2ImgInfo.infoSectPtr + infoSectSize], 0xff, 4);
+    // calc md5 sum
+    tools_md5(&md5buff[0], md5buff.size(), md5sum);
+    return true;
+#endif
+}
