@@ -57,15 +57,18 @@
 #include <string.h>
 #include <signal.h>
 
-
 #ifdef MST_UL
 #include <errno.h>
-#include "mtcr.h"
 #include "mtcr_int_defs.h"
 #endif
 
 #include "mtcr_ib.h"
 
+#ifndef __WIN__
+#include "mtcr_mf.h"
+#include "mtcr_com_defs.h"
+#endif
+
 #include <infiniband/mad.h>
 
 #ifdef __WIN__
@@ -83,7 +86,10 @@
                       return -1;\
                   }\
     } while(0)
+
     #define MY_GetProcAddress(ivm, func_name)  MY_GetPointerAddress(ivm, func_name, f_##func_name)
+    #define MY_GetProcAddressIgnoreFail(ivm, func_name) \
+            ivm->func_name = (f_##func_name)GetProcAddress(ivm->dl_handle, #func_name)
     #define MY_GetVarAddress(ivm, var_name)    MY_GetPointerAddress(ivm, var_name, void*)
 
     #define DLL_HANDLE HINSTANCE
@@ -107,6 +113,10 @@
                           return -1;\
                       } \
     } while(0)
+
+    #define MY_DLSYM_IGNORE_FAIL(mf, func_name) \
+                          mf->func_name = dlsym(mf->dl_handle, #func_name)
+
     #define OP_NOT_SUPPORTED EOPNOTSUPP
     #define IBMAD_CALL_CONV
 
@@ -166,10 +176,8 @@
 #define I2C_DEVICE_ID   0x56
 #define I2C_MEMORY_ADDR 0
 
-#define MELLANOX_OUI0     0x2c9
-#define MELLANOX_OUI1     0xf45214
-#define SWITCHX_DEV_ID    0xc738
-#define CONNECTIB_DEV_ID  0x1011
+#define UNSUPP_DEVS_NUM   15
+#define DEVID_ADDRESS     0xf0014
 
 // Convert BYTES - DWORDS with MEMCPY BE
 #define BYTES_TO_DWORD_BE(dw_dest, byte_src) do {   u_int32_t tmp;\
@@ -193,8 +201,12 @@ typedef uint8_t* IBMAD_CALL_CONV (*f_ib_vendor_call_via)(void *data, ib_portid_t
                                          const struct ibmad_port *srcport);
 typedef uint8_t* IBMAD_CALL_CONV (*f_smp_query_via)(void *buf, ib_portid_t *id, unsigned attrid, unsigned mod,
                                     unsigned timeout, const struct ibmad_port *srcport);
+typedef uint8_t* IBMAD_CALL_CONV (*f_smp_query_status_via)(void *buf, ib_portid_t *id, unsigned attrid, unsigned mod,
+                                    unsigned timeout, int *rstatus, const struct ibmad_port *srcport);
 typedef uint8_t* IBMAD_CALL_CONV (*f_smp_set_via)(void *buf, ib_portid_t *id, unsigned attrid, unsigned mod,
                                   unsigned timeout, const struct ibmad_port *srcport );
+typedef uint8_t* IBMAD_CALL_CONV (*f_smp_set_status_via)(void *buf, ib_portid_t *id, unsigned attrid, unsigned mod,
+                                  unsigned timeout, int *rstatus, const struct ibmad_port *srcport );
 
 typedef void IBMAD_CALL_CONV (*f_mad_rpc_set_retries)(struct ibmad_port *port, int retries);
 typedef void IBMAD_CALL_CONV (*f_mad_rpc_set_timeout)(struct ibmad_port *port, int timeout);
@@ -224,7 +236,9 @@ struct __ibvsmad_hndl_t
     f_ib_vendor_call_via        ib_vendor_call_via;
     f_ib_resolve_portid_str_via ib_resolve_portid_str_via;
     f_smp_query_via             smp_query_via;
+    f_smp_query_status_via      smp_query_status_via;
     f_smp_set_via               smp_set_via;
+    f_smp_set_status_via        smp_set_status_via;
     f_mad_rpc_set_retries       mad_rpc_set_retries;
     f_mad_rpc_set_timeout       mad_rpc_set_timeout;
     f_mad_get_field             mad_get_field;
@@ -246,7 +260,7 @@ static int verbose = 0;
 
 #define IB_SMP_ATTR_CR_ACCESS 0xff50
 #define IB_DATA_INDEX     8
-#define MAX_IB_SMP_DATA_SIZE    IB_SMP_DATA_SIZE - IB_DATA_INDEX
+#define MAX_IB_SMP_DATA_SIZE    (IB_SMP_DATA_SIZE - IB_DATA_INDEX)
 #define MAX_IB_SMP_DATA_DW_NUM  MAX_IB_SMP_DATA_SIZE / 4
 
 static uint64_t
@@ -300,13 +314,13 @@ ibvsmad_craccess_rw_smp(ibvs_mad *h, u_int32_t memory_address, int method, u_int
 }
 
 // TODO: Correct this value.
-#define  MAX_VS_DATA_SIZE IB_VENDOR_RANGE2_DATA_SIZE -  IB_DATA_INDEX
+#define  MAX_VS_DATA_SIZE (IB_VENDOR_RANGE1_DATA_SIZE -  IB_DATA_INDEX)
 #define  MAX_IB_VS_DATA_DW_NUM  MAX_VS_DATA_SIZE / 4
 
 static uint64_t
 ibvsmad_craccess_rw_vs(ibvs_mad *h, u_int32_t memory_address, int method, u_int8_t num_of_dwords, u_int32_t *data)
 {
-    u_int8_t vsmad_data[IB_VENDOR_RANGE2_DATA_SIZE] = {0};
+    u_int8_t vsmad_data[IB_VENDOR_RANGE1_DATA_SIZE] = {0};
     ib_vendor_call_t call;
     int i;
     u_int8_t* p;
@@ -391,7 +405,9 @@ int process_dynamic_linking(ibvs_mad *ivm, int mad_init)
         MY_GetProcAddress(ivm, ib_vendor_call_via       );
         MY_GetProcAddress(ivm, ib_resolve_portid_str_via);
         MY_GetProcAddress(ivm, smp_query_via            );
+        MY_GetProcAddressIgnoreFail(ivm, smp_query_status_via );
         MY_GetProcAddress(ivm, smp_set_via              );
+        MY_GetProcAddressIgnoreFail(ivm, smp_set_status_via );
         MY_GetProcAddress(ivm, mad_rpc_set_retries      );
         MY_GetProcAddress(ivm, mad_rpc_set_timeout      );
         MY_GetProcAddress(ivm, portid2str               );
@@ -427,7 +443,7 @@ int process_dynamic_linking(ibvs_mad *ivm, int mad_init)
         }
     }
     if (!ivm->dl_handle) {
-        const char* errstr = dlerror();
+        const char* errstr=dlerror();
         IBERROR(("%s", errstr));
         //free(ivm);
         return -1;
@@ -441,12 +457,14 @@ int process_dynamic_linking(ibvs_mad *ivm, int mad_init)
     MY_DLSYM(ivm, ib_vendor_call_via       );
     MY_DLSYM(ivm, ib_resolve_portid_str_via);
     MY_DLSYM(ivm, smp_query_via            );
+    MY_DLSYM_IGNORE_FAIL(ivm, smp_query_status_via );
     MY_DLSYM(ivm, smp_set_via              );
+    MY_DLSYM_IGNORE_FAIL(ivm, smp_set_status_via   );
     MY_DLSYM(ivm, mad_rpc_set_retries      );
     MY_DLSYM(ivm, mad_rpc_set_timeout      );
     MY_DLSYM(ivm, mad_get_field            );
-    MY_DLSYM(ivm, portid2str              );
-    MY_DLSYM(ivm, ibdebug               );
+    MY_DLSYM(ivm, portid2str               );
+    MY_DLSYM(ivm, ibdebug                  );
     return 0;
 }
 
@@ -509,28 +527,47 @@ int get_env_vars(ibvs_mad *ivm)
 
 
 
-int is_smp_crspace_supported(ibvs_mad* h)
+int is_vs_crspace_supported(ibvs_mad* h)
 {
     u_int8_t* p;
     u_int8_t mad_data[IB_SMP_DATA_SIZE] = {0};
     u_int32_t dev_id;
-    u_int32_t vend_id;
+    u_int32_t data = 0;
+    int i = 0;
+    u_int32_t unsupported_devs[UNSUPP_DEVS_NUM] = {
+            0x6746, /* MT26438 [ConnectX-2 VPI w/ Virtualization+]"*/
+            0x6764, /* MT26468 [Mountain top]"                     */
+            0xbd34, /* IS4 IB SDR"                                 */
+            0xbd35, /* IS4 IB DDR"                                 */
+            0xbd36, /* IS4 IB QDR"                                 */
+            0xfa66, /* BridgeX VPI up, E/FC down"                  */
+            0xfa7a, /* BridgeX EN up, E/FC down"                   */
+            0x1001, /* ConnectX-2 VF"                              */
+            0x1003, /* MT27500 [ConnectX-3]"                       */
+            0x1005, /* MT27510 Family"                             */
+            0x1007, /* MT27520 ConnectX-3 Pro Family"              */
 
+    };
+    uint64_t ret = ibvsmad_craccess_rw_vs(h, DEVID_ADDRESS, IB_MAD_METHOD_GET, 1, &data);
+    if (!ret) {
+        return 1;
+    }
+    DEBUG(("Vendor Specific is not supported, checking SMP .. "));
     p = h->smp_query_via(mad_data, &h->portid, IB_ATTR_NODE_INFO, 0, 0, h->srcport);
 
     if (NULL == p) {
-        return 0;
+        return 1;
     }
 
     dev_id  = h->mad_get_field(mad_data, 0, IB_NODE_DEVID_F);
-    vend_id = h->mad_get_field(mad_data, 0, IB_NODE_VENDORID_F);
 
-    if ((MELLANOX_OUI0    == vend_id || MELLANOX_OUI1  == vend_id) &&
-        (CONNECTIB_DEV_ID == dev_id  || SWITCHX_DEV_ID == dev_id)) {
-        return 1;
-    } else {
-        return 0;
+    for (i = 0; i < UNSUPP_DEVS_NUM; i++) {
+        if (dev_id == unsupported_devs[i]) {
+            return 1;
+        }
     }
+
+    return 0;
 }
 
 
@@ -550,15 +587,21 @@ int mib_open(const char *name, mfile *mf, int mad_init)
     ib_portid_t *sm_id           = 0;
     int          ca_port         = 0;
     int          dest_type       = IB_DEST_LID;
-    ibvs_mad    *ivm;
+    ibvs_mad    *ivm             = NULL;
     char        *nbuf            = NULL;
     char        *path_str, *p;
     int          rc              = -1;
+    int         lid_provided = 0;
     char        *sl_str;
 
     char* first_comma;
     char* second_comma;
     //ibdebug = 1;
+    if (!mf || !name) {
+        IBERROR(("Bad(null) device argument for inband access"));
+        errno = EINVAL;
+        goto end;
+    }
     if (!(ivm=(ibvs_mad*)malloc(sizeof(ibvs_mad))))
     {
         IBERROR(("can't allocate ibvsmad_mfile"));
@@ -582,11 +625,13 @@ int mib_open(const char *name, mfile *mf, int mad_init)
         dest_type = IB_DEST_LID;
         path_str  = nbuf + 6;
     } else if ((p = strstr(nbuf, "lid-")) != 0) {
+        lid_provided = 1;
         dest_type = IB_DEST_LID;
-		path_str  = p + 4;
+        path_str  = p + 4;
 
     } else if ((p = strstr(nbuf, "lid_noinit-")) != 0) {
-    	dest_type = IB_DEST_LID;
+        lid_provided = 1;
+        dest_type = IB_DEST_LID;
         path_str  = p + 11;
         mad_init = 0;
 
@@ -671,8 +716,14 @@ int mib_open(const char *name, mfile *mf, int mad_init)
 
     // For ConnectIB:
     // If the target device is connectib or SX and we use LID rout, use the SMP LID route and not the GPM
-    if (is_smp_crspace_supported(ivm)) {
-        ivm->use_smp = 1;
+    if (lid_provided) {
+        if (is_vs_crspace_supported(ivm)) {
+            DEBUG(("For this LID using VS MAD"));
+            ivm->use_smp = 0;
+        } else {
+            DEBUG(("For this LID using SMP MAD"));
+            ivm->use_smp = 1;
+        }
     }
 
 #define MTCR_IBSL_ENV "MTCR_IB_SL"
@@ -716,18 +767,22 @@ end:
 int
 mib_close(mfile *mf)
 {
-    // TODO: free the ddl handlers
-	ibvs_mad* h = (ibvs_mad*)(mf->ctx);
-	h->mad_rpc_close_port(h->srcport);
-
-#ifndef IBVSMAD_DLOPEN
-    free_dll_handle(mf);
-#endif
-    free(mf->ctx);
+    if (mf) {
+        if (mf->ctx) {
+        // TODO: free the ddl handlers
+            ibvs_mad* h = (ibvs_mad*)(mf->ctx);
+            h->mad_rpc_close_port(h->srcport);
+    #ifndef IBVSMAD_DLOPEN
+            free_dll_handle(mf);
+    #endif
+            free(mf->ctx);
+            mf->ctx = NULL;
+        }
+    }
     return 0;
 }
 
-#define CHECK_ALLIGN(length) { \
+#define CHECK_ALIGN(length) { \
     if (length % 4) {                                       \
         IBERROR(("Size must be 4 aligned, got %d", length));\
         return -1;                                          \
@@ -737,11 +792,15 @@ mib_close(mfile *mf)
 
 int mib_get_chunk_size(mfile *mf)
 {
+    if (!mf || !mf->ctx) {
+        IBERROR(("get chunk size failed. Null Param."));
+        return -1;
+    }
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
     if (h->use_smp) {
         return MAX_IB_SMP_DATA_SIZE;
     }
-    return IB_VENDOR_RANGE2_DATA_SIZE;
+    return MAX_VS_DATA_SIZE;
 }
 
 /********************************************************
@@ -752,6 +811,10 @@ int mib_get_chunk_size(mfile *mf)
 int
 mib_read4(mfile *mf, u_int32_t memory_address, u_int32_t *data)
 {
+    if (!mf || !mf->ctx || !data) {
+        IBERROR(("cr access read failed. Null Param."));
+        return -1;
+    }
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
 
     DEBUG(("mread4 of 0x%08x", memory_address));
@@ -762,10 +825,6 @@ mib_read4(mfile *mf, u_int32_t memory_address, u_int32_t *data)
     return 4;
 }
 
-
-
-
-
 /********************************************************
 **
 *    Write an IS3 CR-Space 32 bit register
@@ -774,6 +833,10 @@ mib_read4(mfile *mf, u_int32_t memory_address, u_int32_t *data)
 int
 mib_write4(mfile *mf, u_int32_t memory_address, u_int32_t _data)
 {
+    if (!mf || !mf->ctx) {
+        IBERROR(("cr access write failed. Null Param."));
+        return -1;
+    }
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
 
     u_int32_t data = _data;
@@ -788,9 +851,13 @@ mib_write4(mfile *mf, u_int32_t memory_address, u_int32_t _data)
 
 MTCR_API int mib_readblock(mfile *mf, unsigned int offset, u_int32_t *data, int length)
 {
+    if (!mf || !mf->ctx || !data) {
+        IBERROR(("cr access read failed. Null Param."));
+        return -1;
+    }
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
 
-    CHECK_ALLIGN(length);
+    CHECK_ALIGN(length);
 
     if (ibvsmad_craccess_rw(h, offset, IB_MAD_METHOD_GET, (length / 4), data) == ~0ull) {
         IBERROR(("cr access read to %s failed", h->portid2str(&h->portid)));
@@ -801,9 +868,13 @@ MTCR_API int mib_readblock(mfile *mf, unsigned int offset, u_int32_t *data, int
 }
 MTCR_API int mib_writeblock(mfile *mf, unsigned int offset, u_int32_t *data, int length)
 {
+    if (!mf || !mf->ctx || !data) {
+        IBERROR(("cr access write failed. Null Param."));
+        return -1;
+    }
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
 
-    CHECK_ALLIGN(length);
+    CHECK_ALIGN(length);
 
     if (ibvsmad_craccess_rw(h, offset, IB_MAD_METHOD_SET, (length / 4), data) == ~0ull) {
         IBERROR(("cr access read to %s failed", h->portid2str(&h->portid)));
@@ -838,12 +909,18 @@ int is_node_managed(ibvs_mad* h)
 int
 mib_swreset(mfile *mf)
 {
+    if (!mf || !mf->ctx) {
+        errno = EINVAL;
+        IBERROR(("swreset write failed. Null Param."));
+        return -1;
+    }
+
     u_int32_t swreset_timer = 15;
     char* ep;
     char* swreset_env;
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
     u_int8_t* p;
-    u_int8_t vsmad_data[IB_VENDOR_RANGE2_DATA_SIZE] = {0};
+    u_int8_t vsmad_data[IB_VENDOR_RANGE1_DATA_SIZE] = {0};
     ib_vendor_call_t call;
 
     swreset_env = getenv(MTCR_SWRESET_ENV);
@@ -883,14 +960,104 @@ mib_swreset(mfile *mf)
 
 #define IB_SMP_ATTR_REG_ACCESS 0xff52
 
+static int mib_status_translate(int status)
+{
+    switch ((status >> 2) & 0x7) {
+    case 1:
+        return ME_MAD_BAD_VER;
+    case 2:
+        return ME_MAD_METHOD_NOT_SUPP;
+    case 3:
+        return ME_MAD_METHOD_ATTR_COMB_NOT_SUPP;
+    case 7:
+        return ME_MAD_BAD_DATA;
+    }
+
+    if (status & 0x1)
+        return ME_MAD_BUSY;
+    else if ((status >> 1) & 0x1)
+        return ME_MAD_REDIRECT;
+
+    return ME_MAD_GENERAL_ERR;
+}
+
 int mib_acces_reg_mad(mfile *mf, u_int8_t *data)
 {
+    if (!mf || !mf->ctx || !data) {
+        IBERROR(("mib_acces_reg_mad failed. Null Param."));
+        return ME_BAD_PARAMS;
+    }
     u_int8_t* p;
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
+    int status = -1;
+
     // Call smp set function
-    p = h->smp_set_via(data, &(h->portid), IB_SMP_ATTR_REG_ACCESS, 0, 0, h->srcport);
-    if (!p) {
-        return -1;
+    if (h->smp_set_status_via) {
+        p = h->smp_set_status_via(data, &(h->portid), IB_SMP_ATTR_REG_ACCESS, 0, 0, &status, h->srcport);
+    } else {
+        p = h->smp_set_via(data, &(h->portid), IB_SMP_ATTR_REG_ACCESS, 0, 0, h->srcport);
     }
-    return 0;
+    if (!p || status > 0) {
+        if (status != -1) {
+            return mib_status_translate(status);
+        } else {
+            return -1;
+        }
+    }
+
+    return ME_OK;
+}
+
+int mib_smp_set(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod)
+{
+    if (!mf || !mf->ctx || !data) {
+        IBERROR(("mib_smp_set failed. Null Param."));
+        return ME_BAD_PARAMS;
+    }
+    u_int8_t* p;
+    int status = -1;
+    ibvs_mad* h = (ibvs_mad*)(mf->ctx);
+    // Call smp set function
+    if (h->smp_set_status_via) {
+        p = h->smp_set_status_via(data, &(h->portid), attr_id, attr_mod, 0, &status, h->srcport);
+    } else {
+        p = h->smp_set_via(data, &(h->portid), attr_id, attr_mod, 0, h->srcport);
+    }
+    if (!p || status > 0) {
+        if (status != -1) {
+            return mib_status_translate(status);
+        } else {
+            return -1;
+        }
+    }
+
+    return ME_OK;
+}
+
+int mib_smp_get(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod)
+{
+    if (!mf || !mf->ctx || !data) {
+        IBERROR(("mib_smp_get failed. Null Param."));
+        return ME_BAD_PARAMS;
+    }
+    u_int8_t* p;
+    int status = -1;
+    ibvs_mad* h = (ibvs_mad*)(mf->ctx);
+
+    // Call smp set function
+    if (h->smp_query_status_via) {
+        p = h->smp_query_status_via(data, &(h->portid), attr_id, attr_mod, 0, &status, h->srcport);
+    } else {
+        p = h->smp_query_via(data, &(h->portid), attr_id, attr_mod, 0, h->srcport);
+    }
+
+    if (!p || status > 0) {
+        if (status != -1) {
+            return mib_status_translate(status);
+        } else {
+            return -1;
+        }
+    }
+
+    return ME_OK;
 }
